// very simpleminded countedpointer which also allows for
// storage NOT to be deleted. Used in circumstances when the
// caller does not know (want to know) if a temparray or an
// existing array is returned, eg in conversions.
// If you pass in the original array and no conversion needs
// to be done you'll get the original array back.
// If a conversion needs to be done, you may be given a 
// newly allocated array.
// This class does the memory management for you - you still
// have ownership of the original storage and when the
// countedarray goes out of scope it deletes the temp copy if necessary.
//
// IMPORTANT NOTE:
//    As this is meant to be used with *arrays* and not
//    for a pointer to a single object, delete [] is
//    called on the pointer so you must make sure
//    the storage is allocated via new [] !!!!
//
#ifndef PYCASA_COUNTEDARRAY_H
#define PYCASA_COUNTEDARRAY_H


template <typename T>
struct countedarray {
        // default ctor
        countedarray():
            hptr( new holder((T*)0) )
        {}
        // this behaves as normal countedptr
        // the storage will be deleted
        countedarray( T* p ):
            hptr( new holder(p) )
        {}
        // use this c'tor if
        // storage may not be deleted
        // (refcnt will never reach 0)
        // note that the argument is discarded - the
        // value 1234 is.. well.. pfff random?
        countedarray( T* p, int ):
            hptr( new holder(p,1234) )
        {}

        // copy
        countedarray( const countedarray<T>& o ) {
            holder*  optr = hptr;

            o.hptr->refcnt++;
            hptr = o.hptr;

            if( --optr->refcnt==0 ) {
                delete [] optr->ptr;
                delete optr;
            }
        }
        // assign
        const countedarray<T>& operator=( const countedarray<T>& o ) {
            if( this!=&o ) {
                holder*  optr = hptr;

                o.hptr->refcnt++;
                hptr = o.hptr;

                if( --optr->refcnt==0 ) {
                    delete [] optr->ptr;
                    delete optr;
                }
            }
            return *this;
        }

        operator T*( void ) {
            return hptr->ptr;
        }
        operator const T*( void ) const {
            return hptr->ptr;
        }
        ~countedarray() {
            if( --hptr->refcnt==0 ) {
                delete [] hptr->ptr;
                delete hptr;
            }
        }
    private:
        struct holder {
            T*            ptr;
            unsigned int  refcnt;
            holder(T* p):
                ptr( p ),
                refcnt( 1 )
            {}
            holder(T* p, int):
                ptr( p ),
                refcnt( 2 )
            {}
        };
        holder*   hptr;
};

#endif
