#ifndef PYCASA_NUMARRAY_IF_H
#define PYCASA_NUMARRAY_IF_H

// this is numarray specific
#include <nummacro.h>
#include <libnumarray.h>

inline PyArray_TYPES pyarraytype(PyObject* o) {
    return (PyArray_TYPES)(PyArray(o)->descr->type_num);
}

inline int pyarrayndim(PyObject* o) {
    return PyArray(o)->nd;
}

inline int pyarraydim(PyObject* o, int n) {
    return PyArray(o)->dimensions[n];
}

inline int pyarraystride(PyObject* o, int n) {
    return PyArray(o)->strides[n];
}

inline int pyarrayelementsize(PyObject* o) {
    return (PyArray(o)->descr->elsize);
}

inline char* pyarraybytes(PyObject* o) {
    return NA_PTR(PyArray(o));
}

inline char* pyarraygetptr1(PyObject* o, int n) {
    return NA_PTR1(PyArray(o), n);
}

inline bool pyarrayiscontiguous(PyObject* o) {
    return (bool)PyArray_ISCONTIGUOUS(o);
}

inline bool pyarrayisobject(PyObject* o) {
    return pyarraytype(o)==PyArray_OBJECT;
}
// the type specializations

PYCASA_DEFINETYPEMAP(PyArray_CHAR,Int8);

// Numarray uses SBYTE for a numarray of characters
// Set HAVE_PYARRAYSBYTE to non-zero if the types are
// different
#define HAVE_PYARRAYSBYTE 1
PYCASA_DEFINETYPEMAP(PyArray_SBYTE,signed char);
PYCASA_DEFINETYPEMAP(PyArray_INT,Int32);
// On 32bit systems numarray defines PyArray_INT and PyArray_LONG to
// be the same. So we cannot have both TYPEMAPs since
// that would doubly declare the same thing.
// Note:  numarray uses the following condition to
//        decide what the actual type of Long is
#if LP64
#define HAVE_PYARRAYLONG 1
PYCASA_DEFINETYPEMAP(PyArray_LONG,Long);
#else
#define HAVE_PYARRAYLONG 0
#endif
PYCASA_DEFINETYPEMAP(PyArray_UINT,UInt32);
PYCASA_DEFINETYPEMAP(PyArray_OBJECT,PyObject*);
PYCASA_DEFINETYPEMAP(PyArray_DOUBLE,Float64);
PYCASA_DEFINETYPEMAP(PyArray_FLOAT,Float32);
PYCASA_DEFINETYPEMAP(PyArray_CFLOAT,Complex32);
PYCASA_DEFINETYPEMAP(PyArray_CDOUBLE,Complex64);

#define HAVE_PYARRAYSTRING 0

#endif
